' Sensor-Bug mit Entprellung
' Wenn man unmittelbar nach Reset (unterer Anschlag Poti) auf Maximum (oberer Anschlag Poti) dreht,
' wird der Sideswiper-Modus aktiviert. Zurueck in Bug-Mode nach erneutem Reset

' Sprache: Bascom-AVR-Basic Version 2.0.7.9 Demo/Freeware
' Lizenz: WtfPL (https://de.wikipedia.org/wiki/WTFPL)
'
' Version 5: logarithm Mittelwertbildung der Sensor-ADC-Werte
' Version 6: DDRB-Bug behoben
'
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

$regfile = "ATtiny13.dat"
$crystal = 1200000                                          ' Taktfrequenz: 1,2 MHz
$hwstack = 8                                                ' Platz fuer nur 4 Unterprogramme reserviert, spart SRAM
$swstack = 0                                                ' wird nicht gebraucht
$framesize = 0                                              ' dito


Dim Ditavg As Word                                          ' gleitender Mittelwert fuer Dit-Erkennung
Dim Dahavg As Word                                          ' gleitender Mittelwert fuer Dah-Erkennung
Dim Shiftdit As Word                                        ' Hilfsvariable fuer Shift-Befehl
Dim Shiftdah As Word
Dim Anadit As Word                                          ' ADC-Wert fuer Erkennung Dit-Sensor
Dim Anadah As Word
Dim Offtimer As Byte                                        ' Zaehler fuer Abschaltautomatik
Dim Dit As Word                                             ' Punktlaenge
Dim Dit_cnt As Word                                         ' wird bis Punktlaenge gesetzt und dann runtergezaehlt
Dim Ditpart As Word                                         ' halbe Punktlaenge fuer simuliertes Einschwingen des Bug-Punkt-Pendels
Dim Pause As Byte                                           ' Pause=0 : Punkt wird gesendet; Pause=1 : Punktpause
Dim Bugmode As Byte

Const Refana = 128                                          ' ADC-Schwellwert bei Beruehrung des Sensors

Const E = 4                                                 ' Exponentialfaktor (hier 4 also 1/16). Auswirkung bei Änderung:
                                                            ' 1: Ansprechen schon bei Annaherung , 8: verzoegertes Ansprechen

Acsr.acd = 1                                                ' Analog-Komparator ausschalten, spart etwas Strom

Config Adc = Single , Prescaler = Auto                      ' AD-Converter konfigurieren und starten
Start Adc

Ddrb = &B0000_0110                                          ' DDRB.0 ungenutzt; DDRB.1 Tast-Ausgang; DDRB.2 Sensor-ADC-Referenz
Portb = &B000_0000                                          ' PB1=1 wuerde Punkt nach Reset erzeugen (Anzeige Reset-Ende)
Didr0 = &B00111111                                          ' Digitaleingaenge abschalten; spart Strom

On Watchdog Onwdt                                           ' Interrupt-Routine fuer Watchdog-interrupt

' Watchdog als Interrupt-Quelle konfigurieren -      WDCR:  7=WDTIF  6=WDIE  5=WDP3  4=WDCE  3=WDE  2=WDP2  1=WDP1  0=WDP0
Wdtcr = &B0001_1111                                         ' Schreiben auf WDTCR Vorbereiten
Wdtcr = &B0110_0001                                         ' Interrupt statt Reset, Watchdog- Timer auf 8 sec setzen
Sreg.7 = 1                                                  ' Interrupts global freigeben

Waitms 500                                                  ' Vorlaufzeit ADC; auch Wartezeit fuer Erkennung Sideswiper-Modus
Gosub Adclesen

' Rechengang "exponential average":
' Dahavg = Dahavg + Anadah/16 - Dahavg/16   bzw.
' Dahavg = Dahavg + (Anadah * Shift Anadah,Right,4) - Shift Dahavg,Right,4
' Bascom kann das nicht direkt rechnen, daher Zwischenschritt mit Hilfsvariable "Shiftdah" erforderlich

Do
    Do
        GoSub DahSensor                                     ' ADC-Wert abholen
        Shift Anadah, Right, E
        ShiftDah = Dahavg
        Shift Shiftdah, right, E
        Dahavg = Dahavg + Anadah
        Dahavg = Dahavg - Shiftdah
        If Dahavg > Refana Then
            Offtimer = 0                                    ' Abschalt-Timer zuruecksetzen
            PortB.1 = 1                                     ' Strich soll rausgehen
        Else
            PortB.1 = 0
        End If
    Loop until PortB.1 = 0                                  ' Refana unterschritten


    Do
        GoSub DitSensor                                     ' ADC-Wert abholen
        Shift Anadit, Right, E
        ShiftDit = Ditavg
        Shift Shiftdit, right, E
        Ditavg = Ditavg + Anadit
        Ditavg = Ditavg - Shiftdit
        If Ditavg > Refana Then
            If Bugmode = 1 Then                             ' Bug Mode
                If Dit_cnt <= 1 Then                        ' wenn Dit_cnt runtergezaehlt
                    Gosub ADClesen                          ' ggf. neue Speed abholen
                    Dit_cnt = Dit                           ' Dit_cnt wieder auf Maximum setzen
                    Toggle Pause                            ' jedes Mal zwischen Punkt und Pause zwischen Punkten umschalten
                End If
                If Pause = 0 Then                           ' Punkt soll rausgehen
                    PortB.1 = 1
                Else                                        ' also waehrend Punkt-Pause
                    PortB.1 = 0
                End If
                Decr Dit_cnt
            Else                                            ' Sideswiper-Modus
                PortB.1 = 1                                 ' bewirkt Dauerton statt Punkten
            End If

        End If
        If Ditavg < Refana Then                             ' unterer Hysteresewert unterschritten
            PortB.1 = 0
            Dit_cnt = Ditpart                               ' simuliert Einschwingen des Punkt-Pendels
            Pause = 255                                     ' uebler workaround, Teil des o.a. Einschwingens. 255 bewirkt,
            End If                                          '    dass Pause nur ein Mal auf ungleich 1 (entspr. 0) gesetzt wird
        Loop until Ditavg < Refana
        PortB.1 = 0
Loop


Dahsensor:
    Admux = 1                                               ' ADC1 (Referenz) an ADC-Eingang legen ; der muss auf Low liegen (bereits zu Anfang fest konfiguriert)
    Portb.4 = 1                                             ' Pullup an Strichsensor ein
    Ddrb.4 = 1                                              ' Strichsensor nun Ausgang  -->  Sensorpin laden
    Waitus 32
    Portb.4 = 0                                             ' Pullup an Strichsensor aus
    Ddrb.4 = 0                                              ' Strichsensor wieder Eingang --> Sensorpin floatet
    Anadah = Getadc(2)                                      ' Strichsensor auslesen
Return

Ditsensor:
    Admux = 1                                               ' ADC1 (Referenz) an ADC-Eingang legen
    Portb.3 = 1                                             ' Pullup an Punktsensor ein
    Ddrb.3 = 1                                              ' Punktsensor ist nun Ausgang
    Waitus 32
    Portb.3 = 0                                             ' Pullup an Punktsensor aus
    Ddrb.3 = 0                                              ' Punktsensor wieder Eingang
    Anadit = Getadc(3)                                      ' Punktsensor auslesen
Return


Adclesen:
    Dit = Getadc(0)                                         ' Poti an ADC1/PB22
    If Dit < 960 Then
        Bugmode = 1
    End If
    Shift Dit , Right, 2                                    ' Teilung / 4 -> Werte nun bis 255
    Dit = Dit + 32                                          ' nun bis 287
    Dit = 384 - Dit                                         ' nutzbarer Bereich nun: 12 - 28 WpM
'    Dit = 360 - Dit                                         ' nutzbarer Bereich nun: 14 - 38 WpM
    Ditpart = Dit                                           ' wird genutzt fuer fuer simuliertes Einschwingen des Punktpendels
    Shift Ditpart , Right, 2                                ' Teilung / 4
    Ditpart = Ditpart + 32
Return


Onwdt:
    Incr Offtimer                                           ' alle 8 sec wird Offtimer incrementiert
    If Offtimer > 126 Then                                  ' nach 127 * 8 sec. (17 Min) erfolgt Powerdown
        Stop Adc
        DDRB = 0
        Mcucr = &B0011_0000                                 ' Bit 4=1 und Bit 3=0: Powerdown     Bit 5=1: Sleep enable
        !Sleep                                              ' Assembler-Befehl statt Basic-Befehl "Powerdown"; spart ein paar Bytes
    End If
Return

End
